/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.fabric3.timer.quartz;

import org.quartz.spi.JobFactory;

/**
 * JobFactory that allows Runnable instances to be used to execute jobs.
 *
 * @version $Revision$ $Date$
 */
public interface RunnableJobFactory extends JobFactory {

    /**
     * Register the holder that contains the Runnable to be executed.
     *
     * @param holder the holder
     */
    void register(RunnableHolder<?> holder);

    /**
     * Removes a registered holder.
     *
     * @param id the id of the holder
     * @return the holder or null if no holder is registered
     */
    RunnableHolder<?> remove(String id);
}
